# Copyright 2023 Emmanuel Chaboud
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from pathlib import PurePath

from .unified_path import UnifiedPath


class UnifiedPathEncoder(json.JSONEncoder):
    """A JSON encoder that serializes a Path to the string corresponding to the unified representation,
    that is OS-independent.
    """

    def default(self, obj):
        if isinstance(obj, PurePath):
            return UnifiedPath(obj).as_unified()

        return json.JSONEncoder.default(self, obj)
